function [A_est,ind_est,N_est] = Lq_SD_SOMP_mod(X,varargin )
% (X,maxiter,q,E,DR,N_max,stop_crit)
% This is the L_q SD-SOMP ALGORITHM with the stopping criteron

% =============Input==============
% X: hyperspectral image: MxL matrix; M is the spectral dimension, and L is
% the number of pixels


% parameters:

% N_known: the number of endmembers (if known)
% q: 1< q <=inf; default = inf
% DR: the dimension reduction method
%     DR = 'AffineSetFit': affine set fitting-based dimension reduction;
%     DR = 'SVD': SVD based dimension reduction
%     DR = 'none': no dimension reduction
% N_max: the data dimension after dimension reduction
% stop_crit: stop_crit == 'auto' or 'fix_number'
% N_known: you can use N_known, which makes the algorithm identical to SPA


%============== output===============
% A_est: the estimated endmembers
% ind_est: the index set of pure pixels
% N_est: the estimated number of endmembers


% this algorithm needs to call cvx 
% and also the estNoise function, provided by Prof. Jose M Bioucas-Dias


% copyright: Xiao Fu (xfu@umn.edu; xfu@ee.cuhk.edu.hk)
% the algorithm can be found in the paper
% X. Fu, W.-K. Ma, T.-H. Chan, and J. M. Bioucas-Dias, 
% "Self-dictionary sparse regression for hyperspectral unmixing: Greedy pursuit and pure pixel search are related,"  IEEE Journal of Selected Topics on Signal Processing,


%% test for number of required parametres

if (nargin-length(varargin)) ~= 1
    error('Wrong number of required parameters');
end


%--------------------------------------------------------------
% Set the defaults for the optional parameters
%--------------------------------------------------------------

DR = 'AffineSetFit';
q = inf;
E = estNoise(X,'off'); 
N_max = 50;
stop_crit = 'auto';
N_known =  [];

%--------------------------------------------------------------
% Read the optional parameters
%--------------------------------------------------------------
if (rem(length(varargin),2)==1)
    error('Optional parameters should always go by pairs');
else
    for i=1:2:(length(varargin)-1)
        switch upper(varargin{i})
            case 'DR'
                DR = varargin{i+1};
            case 'Q'
                q = varargin{i+1};
            case 'N_MAX'
                N_max = varargin{i+1};
            case 'N_KNOWN'
                N_known = varargin{i+1};
            case 'STOP_CRIT'
                stop_crit = varargin{i+1};
            otherwise
                % Hmmm, something wrong with the parameter string
                error(['Unrecognized option: ''' varargin{i} '''']);
        end;
    end;
end


%% determine wether or not use dimension reduction
switch  DR
    case 'AffineSetFit'
    [M,L] = size(X);
    d = mean(X,2);
    U = X-d*ones(1,L);    
    R_true = (U*U')./L; 
    [eV D] = eig(R_true);
    C = eV(:,end:-1:M-(N_max-2));
    X_r = C'*U;
    X=[X_r;ones(1,L)];
    Y=X;
    E = C'*E;
    case 'SVD'
    [M,L] = size(X);
    U=X;
    R_true = (U*U'/L) ; 
    [eV D] = eig(R_true);
    C = eV(:,end:-1:M-(N_max-1));
    X_r = C'*U;
    X=[X_r];
    Y=X;
    E = C'*E;  
    case 'none'
    Y = X;
end


switch stop_crit
    case 'fix_number'
        if isempty(N_known)==1
            error(['fix_number stopping criterion should be used with N_known']);
        else
            maxiter = N_known;
        end
    case 'auto'
        maxiter = N_max;
end


%% ----------------------- To begin the algorithm ----------------------
[M,L] = size(Y); 
P = eye(M);
A_est = [];
ind_est = [];



for iter = 1:maxiter
    if q~=inf
        [wu,ind] = max((sum(abs(Y'*P*Y).^q)).^(1/q));
    else
        [wuyong,ind] = max(sum(abs(P*Y).^2).^(1/2));
    end
    A_est = [A_est Y(:,ind)];
    P = eye(M) - A_est*pinv(A_est);
    ind_est = [ind_est,ind];
    
    
    %---------------- Stopping Criterion 2----------------
    switch stop_crit
        case 'auto'
        if iter>=2;
            
            E_en = sum(E.^2);
            
            lambda =max(E_en);
            
            cvx_begin quiet
            variable theta(iter-1)
            minimize(  norm(A_est(:,iter)-A_est(:,1:iter-1)*theta,2) )
            subject to
            theta>=0;
            ones(1,iter-1)*theta == 1;
            cvx_end
            
            
            if norm(A_est(:,iter)-A_est(:,1:iter-1)*theta,2)<=2*sqrt(lambda);
                break;
            end
            
        end
        
        case 'fix_number'
           N_est = N_known;        
    end
    %---------------- Stopping Criterion End -------------
    
    
    
end

%% ----  To estimate A_est -----------------
[muyouyong,N_est]=size(A_est);

switch stop_crit
    case 'auto'
        ind_est = ind_est(1:end-1);
        N_est = N_est - 1;
    case 'fix_number'
        ind_est = ind_est;
        N_est = N_est;        
end
A_est = A_est(:,1:N_est);


switch  DR
    case 'AffineSetFit'
        A_est = C*A_est(1:end-1,:) + d*ones(1,N_est);
    case 'SVD'
        A_est =  C*A_est;
    case 'none'
        A_est =  A_est;
end

  